using System;
using UnityEngine;
using UnityEngine.InputSystem;

public class BallBehavior : MonoBehaviour
{
    private PlayerInput _playerInput;
    private InputAction _horizontalAction;
    private float _horizontalAxis;
    public int moveForwardSpeed = 7;
    public float keyboardMoveSpeed = 15f;
    private int _isKeyboard;
    private GameObject _ball;
    private float _ballRadius;
    private Vector3 _ballBottomPos;
    private Boolean _gameOver = false;
    private Animator _animator;


    // Ground Checks
    [Header("Ground Checks")] [SerializeField]
    private bool isGroundedOnAuthorizedLayer;

    [SerializeField] private float groundHeight = 0.3f;
    [SerializeField] private float groundCheckDistance = 0.17f;
    [SerializeField] private LayerMask groundMask = 0;

    [Header("Sounds")] private AudioSource _audioSource;
    [SerializeField] private AudioClip[] _bounceSound;
    [SerializeField] private AudioClip[] _waterSplashSound;

    void Awake()
    {
        _playerInput = GetComponent<PlayerInput>();
        _horizontalAction = _playerInput.currentActionMap.FindAction("Horizontal");
        _horizontalAction.performed += MoveActionPerformed;
        _horizontalAction.canceled += _ => _horizontalAxis = 0;
    }

    void Start()
    {
        _ball = GameObject.Find("Ball");
        _ballRadius = _ball.GetComponent<SphereCollider>().radius;
        _animator = GetComponent<Animator>();
        _audioSource = GetComponent<AudioSource>();
    }

    void Update()
    {
        if (!_gameOver)
        {
            var ballPosition = transform.position;
            var deltaTime = Time.deltaTime;
            ballPosition += Vector3.forward * (moveForwardSpeed * deltaTime);
            ballPosition += Vector3.right * (deltaTime * _horizontalAxis);
            transform.position = ballPosition;
        }

        _ballBottomPos = _ball.transform.position + Vector3.down * (_ballRadius * _ball.transform.localScale.y);

        isGroundedOnAuthorizedLayer = Physics.CheckSphere(_ballBottomPos, groundCheckDistance, groundMask);

        if (_ball.transform.position.y < groundHeight && !isGroundedOnAuthorizedLayer && !_gameOver)
        {
            PlaySound("waterSplash");
            Debug.Log("Game Over: " + _ball.transform.position.y + " " + isGroundedOnAuthorizedLayer);
            GameOver();
        }
        else if (isGroundedOnAuthorizedLayer && !_gameOver)
        {
            PlaySound("bounce");
        }
    }


    private void MoveActionPerformed(InputAction.CallbackContext context)
    {
        if (context.control.device.displayName.Contains("Keyboard"))
        {
            _isKeyboard = 1;
        }
        else
        {
            _isKeyboard = 0;
        }

        _horizontalAxis = context.ReadValue<float>() * (1 + keyboardMoveSpeed * _isKeyboard);
    }

    void OnDrawGizmosSelected()
    {
        // Dessine une boule rouge pour visualier goundCheckDistance
        Gizmos.color = Color.red;
        Gizmos.DrawSphere(_ballBottomPos, groundCheckDistance);
    }

    public void RestartScene()
    {
        UnityEngine.SceneManagement.SceneManager.LoadScene(0);
    }

    private void OnTriggerEnter(Collider other)
    {
        // If it's an obstacle it's a game over
        if (other.gameObject.CompareTag("Obstacle"))
        {
            Debug.Log("Game Over: " + other.gameObject.name);
            GameOver();
        }
    }

    private void GameOver()
    {
        _gameOver = true;
        _animator.enabled = false;
    }

    private void PlaySound(string soundType)
    {
        if (soundType == "bounce")
        {
            _audioSource.PlayOneShot(_bounceSound[UnityEngine.Random.Range(0, _bounceSound.Length)]);
        }
        else if (soundType == "waterSplash")
        {
            _audioSource.PlayOneShot(_waterSplashSound[UnityEngine.Random.Range(0, _waterSplashSound.Length)]);
        }
    }
}